@@ -24,26 +24,27 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "presentwindowsconfig.h"
 #include <kactioncollection.h>
 #include <kaction.h>
-#include <klocale.h>
+#include <KDE/KGlobal>
+#include <KDE/KIcon>
+#include <KDE/KLocalizedString>
+#include <KDE/KStandardDirs>
 #include <kdebug.h>
 #include <kglobalsettings.h>
+#include <kdeclarative.h>
 
 #include <kwinglutils.h>
 
 #include <QMouseEvent>
-#include <QtGui/QPainter>
-#include <QtGui/QGraphicsLinearLayout>
-#include <Plasma/FrameSvg>
-#include <Plasma/PushButton>
-#include <Plasma/Theme>
-#include <Plasma/WindowEffects>
 #include <netwm_def.h>
 
 #include <math.h>
 #include <assert.h>
 #include <limits.h>
 #include <QApplication>
+#include <QDeclarativeContext>
+#include <QDeclarativeEngine>
 #include <QDesktopWidget>
+#include <QGraphicsObject>
 #include <QTimer>
 #include <QVector2D>
 #include <QVector4D>
@@ -70,15 +71,8 @@ PresentWindowsEffect::PresentWindowsEffect()
     , m_highlightedDropTarget(NULL)
     , m_dragToClose(false)
 {
-    m_atomDesktop = XInternAtom(display(), "_KDE_PRESENT_WINDOWS_DESKTOP", False);
-    m_atomWindows = XInternAtom(display(), "_KDE_PRESENT_WINDOWS_GROUP", False);
-    effects->registerPropertyType(m_atomDesktop, true);
-    effects->registerPropertyType(m_atomWindows, true);
-
-    // Announce support by creating a dummy version on the root window
-    unsigned char dummy = 0;
-    XChangeProperty(display(), rootWindow(), m_atomDesktop, m_atomDesktop, 8, PropModeReplace, &dummy, 1);
-    XChangeProperty(display(), rootWindow(), m_atomWindows, m_atomWindows, 8, PropModeReplace, &dummy, 1);
+    m_atomDesktop = effects->announceSupportProperty("_KDE_PRESENT_WINDOWS_DESKTOP", this);
+    m_atomWindows = effects->announceSupportProperty("_KDE_PRESENT_WINDOWS_GROUP", this);
 
     KActionCollection* actionCollection = new KActionCollection(this);
     KAction* a = (KAction*)actionCollection->addAction("Expose");
@@ -111,16 +105,6 @@ PresentWindowsEffect::PresentWindowsEffect()
 
 PresentWindowsEffect::~PresentWindowsEffect()
 {
-    XDeleteProperty(display(), rootWindow(), m_atomDesktop);
-    effects->registerPropertyType(m_atomDesktop, false);
-    XDeleteProperty(display(), rootWindow(), m_atomWindows);
-    effects->registerPropertyType(m_atomWindows, false);
-    foreach (ElectricBorder border, m_borderActivate) {
-        effects->unreserveElectricBorder(border);
-    }
-    foreach (ElectricBorder border, m_borderActivateAll) {
-        effects->unreserveElectricBorder(border);
-    }
     delete m_filterFrame;
     delete m_closeView;
 }
@@ -129,24 +113,24 @@ void PresentWindowsEffect::reconfigure(ReconfigureFlags)
 {
     PresentWindowsConfig::self()->readConfig();
     foreach (ElectricBorder border, m_borderActivate) {
-        effects->unreserveElectricBorder(border);
+        effects->unreserveElectricBorder(border, this);
     }
     foreach (ElectricBorder border, m_borderActivateAll) {
-        effects->unreserveElectricBorder(border);
+        effects->unreserveElectricBorder(border, this);
     }
     m_borderActivate.clear();
     m_borderActivateAll.clear();
     foreach (int i, PresentWindowsConfig::borderActivate()) {
         m_borderActivate.append(ElectricBorder(i));
-        effects->reserveElectricBorder(ElectricBorder(i));
+        effects->reserveElectricBorder(ElectricBorder(i), this);
     }
     foreach (int i, PresentWindowsConfig::borderActivateAll()) {
         m_borderActivateAll.append(ElectricBorder(i));
-        effects->reserveElectricBorder(ElectricBorder(i));
+        effects->reserveElectricBorder(ElectricBorder(i), this);
     }
     foreach (int i, PresentWindowsConfig::borderActivateClass()) {
         m_borderActivateClass.append(ElectricBorder(i));
-        effects->reserveElectricBorder(ElectricBorder(i));
+        effects->reserveElectricBorder(ElectricBorder(i), this);
     }
     m_layoutMode = PresentWindowsConfig::layoutMode();
     m_showCaptions = PresentWindowsConfig::drawWindowCaptions();
@@ -530,11 +514,8 @@ bool PresentWindowsEffect::borderActivated(ElectricBorder border)
     return true;
 }
 
-void PresentWindowsEffect::windowInputMouseEvent(Window w, QEvent *e)
+void PresentWindowsEffect::windowInputMouseEvent(QEvent *e)
 {
-    assert(w == m_input);
-    Q_UNUSED(w);
-
     QMouseEvent* me = static_cast< QMouseEvent* >(e);
     if (m_closeView && m_closeView->geometry().contains(me->pos())) {
         if (!m_closeView->isVisible()) {
@@ -590,7 +571,7 @@ void PresentWindowsEffect::windowInputMouseEvent(Window w, QEvent *e)
                     m_highlightedDropTarget = NULL;
                 }
                 effects->addRepaintFull();
-                XDefineCursor(display(), m_input, QCursor(Qt::PointingHandCursor).handle());
+                effects->defineCursor(Qt::PointingHandCursor);
                 return;
             }
             if (hovering) {
@@ -629,7 +610,7 @@ void PresentWindowsEffect::windowInputMouseEvent(Window w, QEvent *e)
             m_highlightedDropTarget->setIcon(icon.pixmap(QSize(128, 128), QIcon::Normal));
             m_highlightedDropTarget = NULL;
         }
-        XDefineCursor(display(), m_input, QCursor(Qt::PointingHandCursor).handle());
+        effects->defineCursor(Qt::PointingHandCursor);
     } else if (e->type() == QEvent::MouseButtonPress && me->button() == Qt::LeftButton && hovering && m_dragToClose) {
         m_dragStart = me->pos();
         m_dragWindow = m_highlightedWindow;
@@ -641,7 +622,7 @@ void PresentWindowsEffect::windowInputMouseEvent(Window w, QEvent *e)
     if (e->type() == QEvent::MouseMove && m_dragWindow) {
         if ((me->pos() - m_dragStart).manhattanLength() > KGlobalSettings::dndEventDelay() && !m_dragInProgress) {
             m_dragInProgress = true;
-            XDefineCursor(display(), m_input, QCursor(Qt::ForbiddenCursor).handle());
+            effects->defineCursor(Qt::ForbiddenCursor);
         }
         if (!m_dragInProgress) {
             return;
@@ -659,13 +640,13 @@ void PresentWindowsEffect::windowInputMouseEvent(Window w, QEvent *e)
             KIcon icon("user-trash");
             effects->addRepaint(m_highlightedDropTarget->geometry());
             m_highlightedDropTarget->setIcon(icon.pixmap(QSize(128, 128), QIcon::Active));
-            XDefineCursor(display(), m_input, QCursor(Qt::DragMoveCursor).handle());
+            effects->defineCursor(Qt::DragMoveCursor);
         } else if (!target && m_highlightedDropTarget) {
             KIcon icon("user-trash");
             effects->addRepaint(m_highlightedDropTarget->geometry());
             m_highlightedDropTarget->setIcon(icon.pixmap(QSize(128, 128), QIcon::Normal));
             m_highlightedDropTarget = NULL;
-            XDefineCursor(display(), m_input, QCursor(Qt::ForbiddenCursor).handle());
+            effects->defineCursor(Qt::ForbiddenCursor);
         }
     }
 }
@@ -701,11 +682,6 @@ void PresentWindowsEffect::mouseActionWindow(WindowMouseAction& action)
                 m_highlightedWindow->minimize();
         }
         break;
-    case WindowCloseAction:
-        if (m_highlightedWindow) {
-            m_highlightedWindow->closeWindow();
-        }
-        break;
     default:
         break;
     }
@@ -1554,7 +1530,7 @@ void PresentWindowsEffect::setActive(bool active)
         }
 
         // Create temporary input window to catch mouse events
-        m_input = effects->createFullScreenInputWindow(this, Qt::PointingHandCursor);
+        effects->startMouseInterception(this, Qt::PointingHandCursor);
         m_hasKeyboardGrab = effects->grabKeyboard(this);
         effects->setActiveFullScreenEffect(this);
 
@@ -1597,7 +1573,7 @@ void PresentWindowsEffect::setActive(bool active)
         m_windowFilter.clear();
         m_selectedWindows.clear();
 
-        effects->destroyInputWindow(m_input);
+        effects->stopMouseInterception(this);
         if (m_hasKeyboardGrab)
             effects->ungrabKeyboard();
         m_hasKeyboardGrab = false;
@@ -1962,51 +1938,29 @@ void PresentWindowsEffect::screenCountChanged()
 /************************************************
 * CloseWindowView
 ************************************************/
-CloseWindowView::CloseWindowView(QWidget* parent)
-    : QGraphicsView(parent)
+CloseWindowView::CloseWindowView(QWidget *parent)
+    : QDeclarativeView(parent)
     , m_armTimer(new QTimer(this))
 {
     setWindowFlags(Qt::X11BypassWindowManagerHint);
     setAttribute(Qt::WA_TranslucentBackground);
-    setFrameShape(QFrame::NoFrame);
     QPalette pal = palette();
     pal.setColor(backgroundRole(), Qt::transparent);
     setPalette(pal);
-    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-
-    // setup the scene
-    QGraphicsScene* scene = new QGraphicsScene(this);
-    m_closeButton = new Plasma::PushButton();
-    m_closeButton->setIcon(KIcon("window-close"));
-    scene->addItem(m_closeButton);
-    connect(m_closeButton, SIGNAL(clicked()), SIGNAL(close()));
-
-    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout;
-    layout->addItem(m_closeButton);
-
-    QGraphicsWidget *form = new QGraphicsWidget;
-    form->setLayout(layout);
-    form->setGeometry(0, 0, 32, 32);
-    scene->addItem(form);
-
-    m_frame = new Plasma::FrameSvg(this);
-    if (Plasma::Theme::defaultTheme()->currentThemeHasImage("translucent/dialogs/background")) {
-        m_frame->setImagePath("translucent/dialogs/background");
-    } else {
-        m_frame->setImagePath("dialogs/background");
+    foreach (const QString &importPath, KGlobal::dirs()->findDirs("module", "imports")) {
+        engine()->addImportPath(importPath);
+    }
+    KDeclarative kdeclarative;
+    kdeclarative.setDeclarativeEngine(engine());
+    kdeclarative.initialize();
+    kdeclarative.setupBindings();
+
+    rootContext()->setContextProperty("armed", QVariant(false));
+
+    setSource(QUrl(KStandardDirs::locate("data", QLatin1String("kwin/effects/presentwindows/main.qml"))));
+    if (QObject *item = rootObject()->findChild<QObject*>("closeButton")) {
+        connect(item, SIGNAL(clicked()), SIGNAL(close()));
     }
-    m_frame->setCacheAllRenderedFrames(true);
-    m_frame->setEnabledBorders(Plasma::FrameSvg::AllBorders);
-    qreal left, top, right, bottom;
-    m_frame->getMargins(left, top, right, bottom);
-    qreal width = form->size().width() + left + right;
-    qreal height = form->size().height() + top + bottom;
-    m_frame->resizeFrame(QSizeF(width, height));
-    Plasma::WindowEffects::enableBlurBehind(winId(), true, m_frame->mask());
-    form->setPos(left, top);
-    scene->setSceneRect(QRectF(QPointF(0, 0), QSizeF(width, height)));
-    setScene(scene);
 
     // setup the timer - attempt to prevent accidental clicks
     m_armTimer->setSingleShot(true);
@@ -2014,9 +1968,9 @@ CloseWindowView::CloseWindowView(QWidget* parent)
     connect(m_armTimer, SIGNAL(timeout()), SLOT(arm()));
 }
 
-void CloseWindowView::windowInputMouseEvent(QMouseEvent* e)
+void CloseWindowView::windowInputMouseEvent(QMouseEvent *e)
 {
-    if (!isEnabled())
+    if (m_armTimer->isActive())
         return;
     if (e->type() == QEvent::MouseMove) {
         mouseMoveEvent(e);
@@ -2029,25 +1983,17 @@ void CloseWindowView::windowInputMouseEvent(QMouseEvent* e)
     }
 }
 
-void CloseWindowView::drawBackground(QPainter* painter, const QRectF& rect)
-{
-    Q_UNUSED(rect)
-    painter->setRenderHint(QPainter::Antialiasing);
-    m_frame->paintFrame(painter);
-}
-
 void CloseWindowView::arm()
 {
-    setEnabled(true);
+    rootContext()->setContextProperty("armed", QVariant(true));
 }
 
 void CloseWindowView::disarm()
 {
-    setEnabled(false);
+    rootContext()->setContextProperty("armed", QVariant(false));
     m_armTimer->start();
 }
 
-
 } // namespace
 
 #include "presentwindows.moc"
