@@ -29,9 +29,11 @@
 #include <KPluginFactory>
 #include <KAuth/Action>
 
+#include "xrandrx11helper.h"
 #include "xrandrbrightness.h"
 #include "upowersuspendjob.h"
 #include "login1suspendjob.h"
+#include "udevqt.h"
 
 #define HELPER_ID "org.kde.powerdevil.backlighthelper"
 
@@ -80,6 +82,31 @@ bool PowerDevilUPowerBackend::isAvailable()
             if (reply.value().contains(UPOWER_SERVICE)) {
                 kDebug() << "UPower was found, activating service...";
                 QDBusConnection::systemBus().interface()->startService(UPOWER_SERVICE);
+                if (!QDBusConnection::systemBus().interface()->isServiceRegistered(UPOWER_SERVICE)) {
+                    // Wait for it
+                    QEventLoop e;
+                    QTimer *timer = new QTimer;
+                    timer->setInterval(10000);
+                    timer->setSingleShot(true);
+
+                    connect(QDBusConnection::systemBus().interface(), SIGNAL(serviceRegistered(QString)),
+                            &e, SLOT(quit()));
+                    connect(timer, SIGNAL(timeout()), &e, SLOT(quit()));
+
+                    timer->start();
+
+                    while (!QDBusConnection::systemBus().interface()->isServiceRegistered(UPOWER_SERVICE)) {
+                        e.exec();
+
+                        if (!timer->isActive()) {
+                            kDebug() << "Activation of UPower timed out. There is likely a problem with your configuration.";
+                            timer->deleteLater();
+                            return false;
+                        }
+                    }
+
+                    timer->deleteLater();
+                }
                 return true;
             } else {
                 kDebug() << "UPower cannot be found on this system.";
@@ -110,9 +137,29 @@ void PowerDevilUPowerBackend::init()
     if (QDBusConnection::systemBus().interface()->isServiceRegistered(LOGIN1_SERVICE)) {
         m_login1Interface = new QDBusInterface(LOGIN1_SERVICE, "/org/freedesktop/login1", "org.freedesktop.login1.Manager", QDBusConnection::systemBus(), this);
     }
+
+    bool screenBrightnessAvailable = false;
     m_upowerInterface = new OrgFreedesktopUPowerInterface(UPOWER_SERVICE, "/org/freedesktop/UPower", QDBusConnection::systemBus(), this);
-    m_kbdBacklight = new OrgFreedesktopUPowerKbdBacklightInterface(UPOWER_SERVICE, "/org/freedesktop/UPower/KbdBacklight", QDBusConnection::systemBus(), this);
     m_brightnessControl = new XRandrBrightness();
+    if (!m_brightnessControl->isSupported()) {
+        kDebug() << "Using helper";
+        KAuth::Action action("org.kde.powerdevil.backlighthelper.syspath");
+        action.setHelperID(HELPER_ID);
+        KAuth::ActionReply reply = action.execute();
+        if (reply.succeeded()) {
+            m_syspath = reply.data()["syspath"].toString();
+            m_syspath = QFileInfo(m_syspath).readLink();
+
+            UdevQt::Client *client =  new UdevQt::Client(QStringList("backlight"), this);
+            connect(client, SIGNAL(deviceChanged(UdevQt::Device)), SLOT(onDeviceChanged(UdevQt::Device)));
+            screenBrightnessAvailable = true;
+        }
+    } else {
+        kDebug() << "Using XRandR";
+        m_randrHelper = new XRandRX11Helper();
+        connect(m_randrHelper, SIGNAL(brightnessChanged()), this, SLOT(slotScreenBrightnessChanged()));
+        screenBrightnessAvailable = true;
+    }
 
     // Capabilities
     setCapabilities(SignalResumeFromSuspend);
@@ -126,14 +173,23 @@ void PowerDevilUPowerBackend::init()
 
     // Brightness Controls available
     BrightnessControlsList controls;
-    controls.insert(QLatin1String("LVDS1"), Screen);
+    if (screenBrightnessAvailable) {
+        controls.insert(QLatin1String("LVDS1"), Screen);
+        m_cachedBrightnessMap.insert(Screen, brightness(Screen));
+        kDebug() << "current screen brightness: " << m_cachedBrightnessMap.value(Screen);
+    }
 
-    if (m_kbdBacklight->isValid())
-        controls.insert(QLatin1String("KBD"), Keyboard);
-
-    if (!controls.isEmpty()) {
-        m_cachedBrightness = brightness(Screen);
-        kDebug() << "current screen brightness: " << m_cachedBrightness;
+    m_kbdBacklight = new OrgFreedesktopUPowerKbdBacklightInterface(UPOWER_SERVICE, "/org/freedesktop/UPower/KbdBacklight", QDBusConnection::systemBus(), this);
+    if (m_kbdBacklight->isValid()) {
+        // Cache max value
+        m_kbdMaxBrightness = m_kbdBacklight->GetMaxBrightness();
+        // TODO Do a proper check if the kbd backlight dbus object exists. But that should work for now ..
+        if (m_kbdMaxBrightness) {
+            controls.insert(QLatin1String("KBD"), Keyboard);
+            m_cachedBrightnessMap.insert(Keyboard, brightness(Keyboard));
+            kDebug() << "current keyboard backlight brightness: " << m_cachedBrightnessMap.value(Keyboard);
+            connect(m_kbdBacklight, SIGNAL(BrightnessChanged(int)), this, SLOT(onKeyboardBrightnessChanged(int)));
+        }
     }
 
     // Supported suspend methods
@@ -196,28 +252,61 @@ void PowerDevilUPowerBackend::init()
     setBackendIsReady(controls, supported);
 }
 
-void PowerDevilUPowerBackend::brightnessKeyPressed(PowerDevil::BackendInterface::BrightnessKeyType type)
+void PowerDevilUPowerBackend::onDeviceChanged(const UdevQt::Device &device)
 {
-    BrightnessControlsList controls = brightnessControlsAvailable();
-    QList<QString> screenControls = controls.keys(Screen);
+    kDebug() << "Udev device changed" << m_syspath << device.sysfsPath();
+    if (device.sysfsPath() != m_syspath) {
+        return;
+    }
 
-    if (screenControls.isEmpty()) {
+    int maxBrightness = device.sysfsProperty("max_brightness").toInt();
+    float newBrightness = device.sysfsProperty("brightness").toInt() * 100 / maxBrightness;
+
+    if (!qFuzzyCompare(newBrightness, m_cachedBrightnessMap[Screen])) {
+        m_cachedBrightnessMap[Screen] = newBrightness;
+        onBrightnessChanged(Screen, m_cachedBrightnessMap[Screen]);
+    }
+}
+
+void PowerDevilUPowerBackend::brightnessKeyPressed(PowerDevil::BackendInterface::BrightnessKeyType type, BrightnessControlType controlType)
+{
+    BrightnessControlsList allControls = brightnessControlsAvailable();
+    QList<QString> controls = allControls.keys(controlType);
+
+    if (controls.isEmpty()) {
         return; // ignore as we are not able to determine the brightness level
     }
 
-    float currentBrightness = brightness(Screen);
+    if (type == Toggle && controlType == Screen) {
+        return; // ignore as we wont toggle the screen off
+    }
 
-    if (qFuzzyCompare(currentBrightness, m_cachedBrightness)) {
+    float currentBrightness = brightness(controlType);
+
+    int step = 10;
+    if (controlType == Keyboard) {
+        // In case the keyboard backlight has only 5 or less possible values,
+        // 10% are not enough to hit the next value. Lets use 30% because
+        // that jumps exactly one value for 2, 3, 4 and 5 possible steps
+        // when rounded.
+        if (m_kbdMaxBrightness < 6) {
+            step = 30;
+        }
+    }
+
+    if (qFuzzyCompare(currentBrightness, m_cachedBrightnessMap.value(controlType))) {
         float newBrightness;
         if (type == Increase) {
-            newBrightness = qMin(100.0f, currentBrightness + 10);
-        } else {
-            newBrightness = qMax(0.0f, currentBrightness - 10);
+            newBrightness = qMin(100.0f, currentBrightness + step);
+        } else if (type == Decrease) {
+            newBrightness = qMax(0.0f, currentBrightness - step);
+        } else { // Toggle On/off
+            newBrightness = currentBrightness > 0 ? 0 : 100;
         }
 
-        setBrightness(newBrightness, Screen);
+        setBrightness(newBrightness, controlType);
     } else {
-        m_cachedBrightness = currentBrightness;
+        m_cachedBrightnessMap[controlType] = currentBrightness;
     }
 }
 
@@ -245,7 +334,7 @@ float PowerDevilUPowerBackend::brightness(PowerDevil::BackendInterface::Brightne
         kDebug() << "Screen brightness: " << result;
     } else if (type == Keyboard) {
         kDebug() << "Kbd backlight brightness: " << m_kbdBacklight->GetBrightness();
-        result = m_kbdBacklight->GetBrightness() / m_kbdBacklight->GetMaxBrightness() * 100;
+        result = 1.0 * m_kbdBacklight->GetBrightness() / m_kbdMaxBrightness * 100;
     }
 
     return result;
@@ -273,20 +362,31 @@ bool PowerDevilUPowerBackend::setBrightness(float brightnessValue, PowerDevil::B
         success = true;
     } else if (type == Keyboard) {
         kDebug() << "set kbd backlight: " << brightnessValue;
-        m_kbdBacklight->SetBrightness(brightnessValue / 100 * m_kbdBacklight->GetMaxBrightness());
+        m_kbdBacklight->SetBrightness(brightnessValue / 100 * m_kbdMaxBrightness);
         success = true;
     }
 
-    if (success) {
-        float newBrightness = brightness(Screen);
-        if (!qFuzzyCompare(newBrightness, m_cachedBrightness)) {
-            m_cachedBrightness = newBrightness;
-            onBrightnessChanged(Screen, m_cachedBrightness);
-        }
-        return true;
-    }
+    return success;
+}
 
-    return false;
+void PowerDevilUPowerBackend::slotScreenBrightnessChanged()
+{
+    float newBrightness = brightness(Screen);
+    kDebug() << "Brightness changed!!";
+    if (!qFuzzyCompare(newBrightness, m_cachedBrightnessMap[Screen])) {
+        m_cachedBrightnessMap[Screen] = newBrightness;
+        onBrightnessChanged(Screen, m_cachedBrightnessMap[Screen]);
+    }
+}
+
+void PowerDevilUPowerBackend::onKeyboardBrightnessChanged(int value)
+{
+    kDebug() << "Keyboard brightness changed!!";
+    float realValue = 1.0 * value / m_kbdMaxBrightness * 100;
+    if (!qFuzzyCompare(realValue, m_cachedBrightnessMap[Keyboard])) {
+        m_cachedBrightnessMap[Keyboard] = realValue;
+        onBrightnessChanged(Keyboard, m_cachedBrightnessMap[Keyboard]);
+    }
 }
 
 KJob* PowerDevilUPowerBackend::suspend(PowerDevil::BackendInterface::SuspendMethod method)
